import shlex
import re

def command(template):
    fields = shlex.split(template)
    def shell(func):
        def handle(*args):
            data = shlex.split(args[-1])
            if fields[0] == data[0]: 
                func(*args, **dict(list(zip(fields[1:], data[1:]))))
        return handle
    return shell

def regcmd(template):
    reg = re.compile(template)
    def shell(func):
        def handle(*args):
            match = reg.match(args[-1])
            if match: 
                func(*args, **match.groupdict())
        return handle
    return shell

    pass






""" 
Overview
========

This plugin uses codepad.org to run inline code and output
the result on the channel. The output is a url unless the length of the output is short enough.

Commands
========

Command: @run lang code
Description: Run inline code on codepad.org.

Example
=======
<\tau> @run python print 'hi'
<\amelia>  hi

"""

import libpad
from quickirc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd

class Codebox(object):
    def __init__(self, server, max_width=512 * 3):
        self.max_width = max_width
        xmap(server, 'CMSG', self.run)

    @regcmd('@run (?P<lang>[^ ]+) (?P<code>.+)')
    def run(self, server, nick, user, host, target, msg, lang, code):
        url, output = libpad.sandbox(code, lang)

        if len(output) <= self.max_width:
            send_msg(server, target, output)
        else:
            send_msg(server, target, url)


install = Codebox










"""

"""

from untwisted.network import core
from os.path import expanduser, join, exists, dirname
from os import mkdir
from shutil import copyfile
import sys

def init():
    dir = join(expanduser('~'), '.amelia')
    rc  = join(dir, 'ameliarc')
    
    if not exists(dir):
        mkdir(dir)
    
    copyfile(join(dirname(__file__), 'ameliarc'), rc)

def run():    
    dir = join(expanduser('~'), '.amelia')
    rc  = join(dir, 'ameliarc')

    if not exists(rc): 
        print('You must call amelia --init.'); return
    ENV = dict()
    exec(compile(open(rc).read(), rc, 'exec'), ENV)
    core.gear.mainloop()
    







"""
Overview
========

Generate latex from basic python expressions.
Meant to be used in ##math at irc.freenode.org

Commands
========

Command: @lax expression
Description: Output latex for the expression.

Example
=======


"""


from quickirc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd
from ameliabot.tools import send_lines
from liblax.parser import run

class LatexBuilder(object):
    def __init__(self, server):
        xmap(server, 'CMSG', self.build)
    
    @regcmd('@lax (?P<exp>.+)$')
    def build(self, server, nick, user, 
                    host, target, msg, exp):

        try:
            data = str(run(exp))
        except Exception as e:
            send_lines(server, target, str(e))
        else:
            send_lines(server, target, data)
      
install = LatexBuilder
















"""
Overview
========

Whenever it matches a combination of letters that appear to be a laught
it makes the bot laught as well.
"""

from quickirc import send_msg
from re import search, compile
from random import choice, randint
from untwisted.network import xmap

class Troll(object):
    def __init__(self, server, letters, min, max):
        self.letters = letters
        self.max     = max
        self.min     = min
        self.pattern = compile('[%s]{%s,%s}' % (letters, min, max))
        xmap(server, 'CMSG', self.check_msg)

    def check_msg(self, server, nick, user, host, target, msg):
        if search(self.pattern, msg): 
            send_msg(server, target, self.make_seq())
        
    def make_seq(self):
        data   = ''
        for ind in range(0, randint(self.min, self.max)):
            data = data + choice(self.letters)
        return data

install = Troll






"""
Overview
========

This plugin retrieves the docstring for each one of the implemented plugins.

Commands
========

Command: @plugins
Description: Show all available plugins.

Command: @doc plugin
Description Show the help for the plugin.

"""

from ameliabot.tools import send_lines
from untwisted.network import xmap
from ameliabot.cmd import command
from pkgutil import iter_modules
import ameliabot.plugins
import os.path

class Help(object):
    def __init__(self, server):
        xmap(server, 'PMSG', self.plugins)
        xmap(server, 'PMSG', self.send_doc)
    
    @command('@plugins')
    def plugins(self, server, nick, user, host, target, msg):
        dir = os.path.dirname(ameliabot.plugins.__file__)
        data = ''
        for _, name, _ in iter_modules([dir]):
            data = '%s %s' % (data, name)
        send_lines(server, nick, data)

    @command('@doc plugin')
    def send_doc(self, server, nick, user, host, target, msg, plugin):
        doc = ''
        code = 'import ameliabot.plugins.%s\ndoc = ameliabot.plugins.%s.__doc__' 
        exec(code % (plugin, plugin))
        send_lines(server, nick, doc)

install = Help







"""
Overview
========

This plugin is used to receive files automatically through dcc.

Usage
=====

Once this plugin is installed and a directory was set in your ameliarc then whatever files
that are sent to the bot it will be saved in directory that was chosen.

"""

from quickirc import *
from os.path import isfile, join
from untwisted.network import xmap
from untwisted.iostd import CLOSE, CONNECT_ERR
from untwisted.iputils import long_to_ip

class Get(object):
    def __init__(self, server, folder):
        self.folder = folder
        xmap(server, 'DCC SEND', self.dcc_get)

    def dcc_get(self, server, xxx_todo_changeme, filename, address, port, size):
    
        (nick, user, host, 
                        target, msg) = xxx_todo_changeme
        path = join(self.folder, filename)

        if isfile(path):      
            send_msg(server, nick, 'File already exists.')
        else:
            fd = open(path, 'wb')
            dccclient = DccClient(long_to_ip(int(address)), 
                                        int(port), fd, int(size)) 
    
            def is_done(dcclient, msg):
                send_msg(server, nick, msg)
                fd.close()
    
            xmap(dccclient, DONE, is_done, 'Done.')
            xmap(dccclient, CLOSE, lambda dccclient, spin, err: is_done(spin, 'Failed.'))
            xmap(dccclient, CONNECT_ERR, lambda dccclient, spin, err: is_done("It couldn't connect."))
    
    
install = Get











"""
Usage: 

# ./amelia/ameliarc

from ameliabot.plugins import antispam

def PLUGIN_SCHEME(server):
    # 1 lines per second, command to be executed if the rate is overriden.
    antispam.install(server, n_lines, n_secs,  cmd='kick {chan} {nick} :Amelia rocks!')

"""

from quickirc import send_cmd, send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd
import time

class AntiSpam(object):
    def __init__(self, server, n_lines, n_secs, cmd):
        """
        The n_lines means the number of lines per n_secs.

        Example for command:
            cmd = 'kick {chan} {nick} :Amelia rocks!'
            cmd = 'chanserv akick {chan} add *!*@{host} : get out!
        """
        xmap(server, 'CMSG', self.watcher)
        self.data    = {}
        self.n_lines = n_lines
        self.n_secs  = n_secs
        self.cmd     = cmd

    def watcher(self, server, nick, user, host, target, msg):
        count, start       = self.data.setdefault(host, [0,  time.time()])
        count              = count + 1
        self.data[host][0] = count

        if time.time() - start > self.n_secs:
            self.data[host][0], self.data[host][1] = 0, time.time()
            if self.n_lines/float(self.n_secs) < count/(time.time() - start):
                send_cmd(server, self.cmd.format(nick=nick, 
                         chan=target, host=host, user=user))

install = AntiSpam
















"""
Overview
========
This plugin uses wolfram alpha to compute mathematical expressions.

Commands
========

Command: @calc expression
Description: Computes the expression using wolfram alpha.

Example
=======

<\tau> @calc integrate x^2
<\amelia>  integral x^2  dx = x^3/3+constant

"""

import wolframalpha

from quickirc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd
from ameliabot.tools import send_lines

class Calculate(object):
    def __init__(self, server, appid):
        self.client = wolframalpha.Client(appid)
        xmap(server, 'CMSG', self.calculate)
    
    @regcmd('@calc (?P<exp>.+)$')
    def calculate(self, server, nick, user, host, target, msg, exp):
        for pod in self.client.query(exp):
            if getattr(pod, 'text', None):
                send_lines(server, target, pod.text)
    
install = Calculate















"""
Overview
========

Used to add a note to a peer when he joins the channel.

Commands
========

Command: @note-add peer data
Description: Add a msg to be shown when a nick joins a channel where the bot is in.

Command: @note-del peer
Description: Remove a note from a nick.
"""

from collections import namedtuple, defaultdict

from untwisted.network import xmap

from quickirc import send_msg
from ameliabot.cmd import command, regcmd

MAX_NOTES_PER_PEER = 2
MAX_NOTES = 10

# source = (nick, user, host)
Message = namedtuple('Message', ('source', 'message'))

class Note(object):
    def __init__(self, server):
        xmap(server, 'CMSG', self.note_add)
        xmap(server, 'CMSG', self.note_del)
        xmap(server, 'JOIN', self.send_note)
        self.base = defaultdict(list)

    @regcmd(r'@note-add\s+(?P<peer>\S+)\s+(?P<data>.*)')
    def note_add(self, server, nick, user, host, target, msg, peer, data):
        msgs = self.base[peer.lower()]
        user_msgs = [m for m in msgs if m.source[1:] == (user, host)
                     or user.startswith('~') and m.source[2] == host]
        if len(user_msgs) + 1 > MAX_NOTES_PER_PEER:
            send_msg(server, target,
                '%s: you may leave no more than %d notes for "%s".' % (
                nick, MAX_NOTES_PER_PEER, peer))
        elif len(msgs) + 1 > MAX_NOTES:
            send_msg(server, target,
                '%s: there are too many notes saved for "%s".' % (
                 nick, peer))
        else:
            msgs.append(Message(
                source=(nick,user,host), message=data))
            self.base[peer.lower()] = msgs
            send_msg(server, target,
            '%s: note for "%s" saved.' % (nick, peer))

    @command('@note-del peer')
    def note_del(self, server, nick, user, host, target, msg, peer):
        peer = peer.lower()
        msgs = self.base.get(peer, [])
        orig_len = len(msgs)
        msgs = [m for m in msgs if m.source[1:] != (user, host)]
        if msgs: self.base[peer] = msgs
        elif peer in self.base: del self.base[peer]
        send_msg(server, target or nick, '%s: %d notes deleted.' % (
            nick, orig_len - len(msgs)))

    def send_note(self, server, nick, user, host, channel):
        if nick.lower() not in self.base: return
        for msg in self.base[nick.lower()]:
            send_msg(server, channel, '%s: <%s> %s' % (
                nick, '%s!%s@%s' % msg.source, msg.message))
        del self.base[nick.lower()]

install = Note












""" 
Overview
========

Used to check last time a peer was seen online.

Commands
========

Command: @seen peer
Description: Show how long a nick has been inactive.

"""

from quickirc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import command
import time

class Seen(object):
    def __init__(self, server):
        self.database = dict()

        xmap(server, 'CMSG', self.record)
        xmap(server, 'CMSG', self.check)

    def record(self, server, nick, user, host, target, msg):
        self.database[nick.lower()] = time.time()

    @command('@seen peer')
    def check(self, server, nick, user, host, target, msg, peer):
        try:
            initial = self.database[peer.lower()] 
        except:
            send_msg(server, target, "No records for that nick.")
        else:
            final   = time.time()
            rate    = final - initial
            hour    = int(rate / (60 ** 2))
            rate    = int(rate % (60 ** 2))
            min     = int(rate / 60)
            sec     = int(rate % 60)

            send_msg(server, target, 
                '%s last seen %s:%s:%s ago.' % (peer, hour, min, sec))



install = Seen





#! /usr/bin/env python

from distutils.core import setup

setup(name="ameliabot",
      version="2.0.0",
      packages=["ameliabot", 
                "ameliabot.plugins",
                'ameliabot.plugins.quote'],
      scripts=['amelia'],
      package_data={'ameliabot': ['ameliarc', '/ameliabot/ameliarc'],
                    'ameliabot.plugins.quote':['quote_database', '/ameliabot/plugins/quote/quote_database']},
      author="Iury O. G. Figueiredo",
      author_email="ioliveira@id.uff.br",
      url='https://github.com/iogf/ameliabot',
      download_url='https://github.com/iogf/ameliabot/releases',
      keywords=['irc', 'ircbot', 'untwisted', 'freenode', 'ameliabot', 'amelia'],
      classifiers=[],
      description="A nifty IRC Bot written on top of untwisted framework")

































from quickirc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd
from liblax.parser import run
import urllib.request, urllib.parse, urllib.error
import urllib.request, urllib.error, urllib.parse

def post(data, title, name):
    """
    """

    URL               = 'http://mathb.in'
    opener            = urllib.request.build_opener()
    opener.addheaders = [('User-agent', 'Mozilla/5.0')]

    head = {'code':'$%s$' % data,
    'title': title,'name': name,
    'submit':'Save and get new URL'}

    ptr = opener.open(URL, urllib.parse.urlencode(head))
    return ptr.geturl()

class Mathb(object):
    def __init__(self, server):
        xmap(server, 'CMSG', self.build)
    
    @regcmd('@mathb (?P<exp>.+)$')
    def build(self, server, nick, user, 
                    host, target, msg, exp):

        try:
            data = str(run(exp))
        except Exception as e:
            send_msg(server, target, str(e))
        else:
            send_msg(server, target, post(data, 
                'Math formulae', 'Ameliabot'))
      
install = Mathb




"""
Overview
========

This plugin show an welcome message whenever an user joins a channel where the bot is in.

"""

from quickirc import send_msg
from untwisted.network import xmap

def install(server):
    xmap(server, 'JOIN', send_welcome)

def send_welcome(server, nick, user, host, channel):
        send_msg(server, channel, 'Welcome to our relaxed place !')













from untwisted.network import Spin, xmap
from untwisted.iostd import Client, Stdin, Stdout, CLOSE, CONNECT_ERR, CONNECT
from untwisted.splits import Terminator, logcon
from quickirc import Irc, CTCP, Misc, send_cmd
from socket import socket, AF_INET, SOCK_STREAM, socket, gethostbyname
from ameliabot import adm
from untwisted.tools import coroutine

def connect(servaddr, port, nick, user, nick_passwd, adm_passwd, chan_list):
    sock   = socket(AF_INET, SOCK_STREAM)
    ip     = gethostbyname(servaddr)
    server = Spin(sock)
    Client(server)

    def auto_join(server, *args):
        send_cmd(server, nick_passwd)
        for ind in chan_list:
            send_cmd(server, 'JOIN %s' % ind)

    @coroutine
    def get_myaddr(server, servaddr, nick, msg):
        server.nick = nick
        send_cmd(server, 'USERHOST %s' % nick)
        args        = yield server, '302'
        _, _, ident = args
        user, myaddr   = ident.split('@')
        server.myaddr  = myaddr

    def update_nick(server, nick_x, user, host, nick_y):
        if server.nick == nick_x:
            server.nick = nick_y
    
    def handle_connect(server):
        Stdin(server)
        Stdout(server)
        Terminator(server)

        Irc(server)
        CTCP(server)

        Misc(server)
        adm.install(server)

        xmap(server, 'PING', lambda server, prefix, servaddr: 
                send_cmd(server, 'PONG :%s' % servaddr))
        
        xmap(server, CLOSE, lambda server, err: lose(server))
        logcon(server)

        xmap(server, '376', auto_join)
        xmap(server, '376', get_myaddr)
        xmap(server, 'NICK', update_nick)

        server.servaddr    = servaddr
        server.port        = port
        server.nick        = nick
        server.user        = user
        server.chan_list   = chan_list
        server.nick_passwd = nick_passwd
        server.adm_passwd  = adm_passwd

        send_cmd(server, 'NICK %s' % nick)
        send_cmd(server, 'USER %s' % user) 
    
    xmap(server, CONNECT, handle_connect)
    xmap(server, CONNECT_ERR, lambda server, err: lose(server))

    server.connect_ex((ip, port))
    return server









from time import asctime
from random import choice, randint
from untwisted.network import xmap
from quickirc import send_msg
from untwisted.timer import Timer
import os
import re

class Advisor(object):
    def __init__(self, server, questions, suggestions, pmed_file='pmed', 
                                            blacklist_file='blacklist', timeout=60 * 3):

        self.questions      = self.load(questions)
        self.suggestions    = self.load(suggestions)
        self.timeout        = timeout
        self.pmed           = self.load(pmed_file)
        self.pmed_file      = pmed_file
        self.blacklist      = self.load(blacklist_file)
        self.blacklist_file = blacklist_file
        xmap(server, 'JOIN', self.send_question)
        xmap(server, 'PMSG', self.check_answer)

    def load(self, filename):
        with open(filename, 'a+') as fd:
            return re.split('\n+', fd.read().strip().lower())

    def check_answer(self, server, nick, user, host, target, msg):
        if (nick in self.pmed or host in self.pmed) and \
                           not (nick in self.blacklist or host in self.blacklist):
            Timer(randint(0, self.timeout), lambda : 
                             send_msg(server, nick, choice(self.suggestions)))
            self.register_user(self.blacklist, self.blacklist_file, nick, host)

    def send_question(self, server, nick, user, host, channel):
        if not (host.lower() in self.pmed or nick.lower() in self.pmed): 
            Timer(randint(0, self.timeout), lambda : 
                            send_msg(server, nick, choice(self.questions)))
            self.register_user(self.pmed, self.pmed_file, nick, host)

    def register_user(self, lst, filename, nick, host):
        lst.append(nick.lower())
        lst.append(host.lower())
        with open(filename, 'a+') as fd:
            fd.write('%s\n%s\n' % (nick, host))

install = Advisor








